==> mpd_0.20.18.orig.tar.xz: mpd-0.20.18/src/pcm/Dsd32.cxx <==
/*
 * Copyright 2003-2017 The Music Player Daemon Project
 * http://www.musicpd.org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "config.h"
#include "Dsd32.hxx"
#include "PcmBuffer.hxx"
#include "util/ConstBuffer.hxx"

/**
 * Construct a 32 bit integer from four bytes.
 */
static constexpr inline uint32_t
Construct32(uint8_t a, uint8_t b, uint8_t c, uint8_t d)
{
	/* "a" is the oldest byte, which must be in the most
	   significant byte */

	return uint32_t(d) | (uint32_t(c) << 8) |
		(uint32_t(b) << 16) | (uint32_t(a) << 24);
}

static constexpr inline uint32_t
Dsd8To32Sample(const uint8_t *src, unsigned channels)
{
	return Construct32(src[0], src[channels],
			   src[2 * channels], src[3 * channels]);
}

ConstBuffer<uint32_t>
Dsd8To32(PcmBuffer &buffer, unsigned channels, ConstBuffer<uint8_t> _src)
{
	const size_t in_frames = _src.size / channels;
	const size_t out_frames = in_frames / 4;
	const size_t out_samples = out_frames * channels;

	const uint8_t *src = _src.data;
	uint32_t *const dest0 = buffer.GetT<uint32_t>(out_samples);
	uint32_t *dest = dest0;

	for (size_t i = 0; i < out_frames; ++i) {
		for (size_t c = 0; c < channels; ++c)
			*dest++ = Dsd8To32Sample(src++, channels);

		src += 3 * channels;
	}

	return {dest0, out_samples};
}

==> mpd_0.21.1.orig.tar.xz: mpd-0.21.1/src/pcm/Dsd32.cxx <==
/*
 * Copyright 2003-2018 The Music Player Daemon Project
 * http://www.musicpd.org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "config.h"
#include "Dsd32.hxx"
#include "PcmBuffer.hxx"
#include "util/ConstBuffer.hxx"

/**
 * Construct a 32 bit integer from four bytes.
 */
static constexpr inline uint32_t
Construct32(uint8_t a, uint8_t b, uint8_t c, uint8_t d) noexcept
{
	/* "a" is the oldest byte, which must be in the most
	   significant byte */

	return uint32_t(d) | (uint32_t(c) << 8) |
		(uint32_t(b) << 16) | (uint32_t(a) << 24);
}

static constexpr inline uint32_t
Dsd8To32Sample(const uint8_t *src, unsigned channels) noexcept
{
	return Construct32(src[0], src[channels],
			   src[2 * channels], src[3 * channels]);
}

ConstBuffer<uint32_t>
Dsd8To32(PcmBuffer &buffer, unsigned channels,
	 ConstBuffer<uint8_t> _src) noexcept
{
	const size_t in_frames = _src.size / channels;
	const size_t out_frames = in_frames / 4;
	const size_t out_samples = out_frames * channels;

	const uint8_t *src = _src.data;
	uint32_t *const dest0 = buffer.GetT<uint32_t>(out_samples);
	uint32_t *dest = dest0;

	for (size_t i = 0; i < out_frames; ++i) {
		for (size_t c = 0; c < channels; ++c)
			*dest++ = Dsd8To32Sample(src++, channels);

		src += 3 * channels;
	}

	return {dest0, out_samples};
}
